class DataFrameHeaders:
    STATUS_TYPE_SUMMARY_HEADER_PREFIX = "status_type_summary."

    PROPERTY_COLUMN_HEADER_PREFIX = "properties."

    CATEGORY_COLUMN_HEADERS = [
        "general",
        "inputs",
        "outputs",
        "data",
        "properties",
    ]
